import tensorflow as tf
#import numpy as np
import cv2
import sys
import os
import scipy.misc
import matplotlib.pyplot as plt

db='test'  #test, train


if (db=='test'):
    image_list_file = '/HDD/disk_A/sensor_recordings/data_clean_hot/data_test.txt'
    image_folder_in = '/HDD/disk_A/sensor_recordings/data_clean_hot/'
    image_folder_out = '/HDD/disk_A/sensor_recordings/data_clean_hot/tf_data/'
    tfrecord_file_base ='validation'
    tfrecord_len= 10000
if(db=='train'):
    image_list_file = '/HDD/disk_A/sensor_recordings/data_clean_hot/data_train.txt'
    image_folder_in = '/HDD/disk_A/sensor_recordings/data_clean_hot/'
    image_folder_out = '/HDD/disk_A/sensor_recordings/data_clean_hot/tf_data/'
    tfrecord_file_base = 'train'
    tfrecord_len = 10000


#########################################################
# Reads the image list file
# image_folder = /path/to/images/
# format of the image_list_file  image.png[tab]label
# label should be an integer
# Returns a image list and the corresponding label list
#########################################################
def read_labeled_image_list(image_folder, image_list_file, lab_type):
    """Reads a .txt file containing pathes and labeles
    Args:
       image_list_file: a .txt file with one /path/to/image per line
       label: optionally, if set label will be pasted after each line
    Returns:
       List with all filenames in file image_list_file
    """
    f = open(image_list_file, 'r')
    filenames = []
    targets = []
    labels = []
    for line in f:
        filename, label = line[:-1].split('\t')
        filenames.append(image_folder+filename)
        target = label
        if (lab_type == 'int'):
            labels.append(int(label))
            targets.append(int(target))
        elif (lab_type == 'float'):
            label = float(label)
            target = label
            if label > 0.0:
                label = 0
            else:
                label = 1
            if line[0:4] == 'left':
                target -= 0.2
            elif line[0:5] == 'right':
                target += 0.2
            elif line[0:6] == 'center':
                pass

            labels.append(label)
            targets.append(target)

    return filenames, labels, targets

########################################################
# Load image
########################################################
def image_size(addr):
    # read an image and resize to (224, 224)
    # cv2 load images as BGR, convert it to RGB
    img = cv2.imread(addr)

    height, width =img.shape[:2]
    return height, width


########################################################
# Convert data into features
########################################################

def _int64_feature(value):
  return tf.train.Feature(int64_list=tf.train.Int64List(value=[value]))

def _bytes_feature(value):
  return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value]))

def _float_feature(value):
  return tf.train.Feature(float_list=tf.train.FloatList(value=[value]))



########################################################
# Write data into tfrecords
########################################################


image_files, labels, targets = read_labeled_image_list(image_folder_in, image_list_file, 'float')
nimages=len(image_files)

j=0

while(1):

    tf_filename =image_folder_out + tfrecord_file_base + str(j)+ '.tfrecord'

    if not os.path.exists(image_folder_out):
        os.makedirs(image_folder_out)

    # open the TFRecords file
    writer = tf.python_io.TFRecordWriter(tf_filename)

    for i in range(tfrecord_len):
        # print how many images are saved every 1000 images
        if not i % 1000:
            print ('Data- {}: {}/{}'.format(db,i, tfrecord_len))
            sys.stdout.flush()

        # Load the image
        current_index=j*tfrecord_len+i
        if (current_index >= nimages):
            j=-1
            break

        img_raw = tf.gfile.FastGFile(image_files[current_index], 'rb').read()
        im_height, im_width = image_size(image_files[current_index])

        label = labels[current_index]
        target = targets[current_index]


        file_index=current_index

        # Create a feature
        feature = {'image/class/label': _int64_feature(int(label)),
                   'image/regress/target': _float_feature(target),
                   'image/index': _int64_feature(file_index),
                   'image/height': _int64_feature(im_height),
                   'image/width': _int64_feature(im_width),
                   'image/format': _bytes_feature(b'png'),
                   'image/encoded': _bytes_feature(tf.compat.as_bytes(img_raw))}

        # Create an example protocol buffer
        example = tf.train.Example(features=tf.train.Features(feature=feature))

        # Serialize to string and write on the file
        writer.write(example.SerializeToString())

    if(j<0):
        break
    else:
        j=j+1

writer.close()
sys.stdout.flush()
