"""
Experimental verification of predicted steering angle with segmented images or road.

"""

import cv2
import numpy as np
import pickle
from matplotlib import pyplot as plt
from curves import Curves
import scipy.misc as sm
import math
import os
import scipy.io

def  perspective_transform(img):

    img_size = (img.shape[1], img.shape[0])
    s = 0
    src = np.float32([(50, 256), (137,150), (183, 150), (270, 256)])
    dst = np.float32([(85, 256), (95+s,0), (225-s, 0), (235, 256)])

    M = cv2.getPerspectiveTransform(src, dst)
    warped = cv2.warpPerspective(img, M, img_size)

    return warped


def make_path(img, steering):

    if steering > 36.0 or steering < -36.0:
        print('Max steering angle possible is 35 degrees.')
        exit()

    steering *= -1.0
    img = cv2.cvtColor(img,cv2.COLOR_GRAY2RGB)

    x_center = int(img.shape[1]/2)
    y_center = img.shape[0]

    offset = math.tan((steering)*math.pi/180.0) * img.shape[0]
    img = cv2.line(img,(x_center, y_center),(x_center+int(offset),0),(255,0,0),2)
    img = cv2.cvtColor(img,cv2.COLOR_RGB2GRAY)

    return img

def smooth(img):

    thres = cv2.threshold(img, 90, 255, cv2.THRESH_BINARY)[1]
    blur = cv2.pyrUp(img)
    for i in range(10):
        blur = cv2.medianBlur(blur, 11)

    blur = cv2.pyrDown(blur)
    img = cv2.threshold(blur, 100, 255, cv2.THRESH_BINARY)[1]

    return img

def identical(img, imgDraw):
    return np.array_equal(img, imgDraw)

def prep(img):
    img[img >= 220] = 255
    img[img < 220] = 0
    img = np.array(img, dtype=np.int16)
    return img

def error(targets,pred_targets):
    errorArray = []
    for i in range(len(targets)):
        if abs(targets[i]-pred_targets[i]) > 6.0:
            errorArray.append(False)
        else:
            errorArray.append(True)
    return errorArray

def compare_lists(verify,errorArray):
    k = 0
    for i in range(len(verify)):
        if verify[i] == errorArray[i]:
            k += 1
    return float(k) / float(len(verify))

def path_verification(verify):
    k = 0
    for i in range(len(verify)):
        if verify[i]:
            k += 1
    return float(k) / float(len(verify))




mat = scipy.io.loadmat('verification_data.mat')
pred_targets = mat['pred_targets']
pred_targets = pred_targets[0]
targets = mat['targets']
targets = targets[0]

errorArray = error(targets, pred_targets)



verify = []

for i in range(len(os.listdir('./verify_images/'))):
    img = cv2.imread('./verify_images/img_{}.png'.format(i),0)
    img = smooth(img)
    img = perspective_transform(img)
    imgDraw = make_path(img,-float(pred_targets[i]))
    img = prep(img)
    imgDraw = prep(imgDraw)
    check = identical(img, imgDraw)
    verify.append(check)

print(verify)
print(errorArray)

print(path_verification(verify))
print(compare_lists(verify,errorArray))
